@@ -65,7 +65,7 @@ static int edit_one_message (CONTEXT *ctx, HEADER *cur)
   omagic = DefaultMagic;
   DefaultMagic = M_MBOX;
 
-  rc = (mx_open_mailbox (tmp, M_APPEND, &tmpctx) == NULL) ? -1 : 0;
+  rc = (mx_open_mailbox (tmp, M_NEWFOLDER, &tmpctx) == NULL) ? -1 : 0;
 
   DefaultMagic = omagic;
 
@@ -88,12 +88,11 @@ static int edit_one_message (CONTEXT *ctx, HEADER *cur)
   if (stat (tmp, &sb) == 0)
     mtime = sb.st_mtime;
 
-  mutt_edit_file ((!Editor || mutt_strcmp ("builtin", Editor) == 0) ? 
-		  NONULL(Visual) : NONULL(Editor), tmp);
+  mutt_edit_file (NONULL(Editor), tmp);
 
   if ((rc = stat (tmp, &sb)) == -1)
   {
-    mutt_error (_("Can't stat: %s"), strerror (errno));
+    mutt_error (_("Can't stat %s: %s"), tmp, strerror (errno));
     goto bail;
   }
   
