@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -218,6 +218,7 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
       if ((pid = mutt_create_filter (cmd, NULL, &fp, NULL)) < 0)
       {
 	dprint (1, (debugfile, "mutt_get_token: unable to fork command: %s", cmd));
+	FREE (&cmd);
 	return (-1);
       }
       FREE (&cmd);
@@ -1201,7 +1202,10 @@ static int parse_set (BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
       mutt_extract_token (tmp, s , 0);
 
       if (parse_sort ((short *) MuttVars[idx].data, tmp->data, map, err) == -1)
+      {
+	r = -1;
 	break;
+      }
     }
     else
     {
@@ -1809,8 +1813,16 @@ void mutt_init (int skip_sys_rc, LIST *commands)
   }
 
   Tempdir = safe_strdup ((p = getenv ("TMPDIR")) ? p : "/tmp");
-  Editor = safe_strdup ((p = getenv ("EDITOR")) ? p : "vi");
-  Visual = safe_strdup ((p = getenv ("VISUAL")) ? p : Editor);
+
+  p = getenv ("VISUAL");
+  if (!p)
+  {
+    p = getenv ("EDITOR");
+    if (!p)
+      p = "vi";
+  }
+  Editor = safe_strdup (p);
+  Visual = safe_strdup (p);
 
   if ((p = getenv ("REPLYTO")) != NULL)
   {
